open D1, "MessierDistances.txt" or die;
while(<D1>) {
    if (/([0-9]+)\t([.e+0-9]+)/) {
        $Distances{"M$1"} = $2;
    }
}
close D1;

open D1, "DistanceData.txt" or die;
while(<D1>) {
    if (/([A-Z0-9]+)\t([.e+0-9]+)/) {
        $Distances{$1} = $2;
    }
}
close D1;


open SAC, "SAC_DeepSky_Ver80_QCQ.TXT" or die;
open OUT, ">$ARGV[0]" or die;

if ( $ARGV[0] =~ /sac([0-9]+)-([0-9]+)/ ) {
    $minMag = $1;
    $maxMag = $2;
    print OUT "Saguaro DSO|$minMag<=m<$maxMag\n";
}
elsif ( $ARGV[0] =~ /sac([0-9]+)-/ ) {
    $minMag = $1;
    $maxMag = 10000;
    print OUT "Saguaro DSO|$minMag<=m\n";
}
elsif ( $ARGV[0] =~ /sac([0-9]+)/ ) {
    $minMag = -10000;
    $maxMag = $1;
    print OUT "Saguaro DSO|m<$maxMag\n";
}
else {
    print OUT "Saguaro DSO\n";
    $minMag = -10000;
    $maxMag = 10000;
}

$_=<SAC>;
s/[\r\n]//g;
@labels = GetLine($_);
$i = 0;
foreach (@labels) {
    $I{$_} = $i++;
}

while(<SAC>) {
    if (!/,/) {
        next;
    }
    s/[\r\n]//g;
    s/\s+/ /g;
    s/\b0([0-9])/$1/g;
    @line = GetLine($_);
    $m = line("MAG");
    $m =~ s/[^-.0-9]//g;
    if ( $m < $minMag || $maxMag <= $m ) {
        next;
    }
    $distance = -1;
    $object = line("OBJECT");
    $object =~ s/\s//g;
    if(defined($Distances{$object})) {
        $distance = $Distances{$object};
    }
    else {
        $object = line("OTHER");
        if ($object =~ /\bM\s*([0-9]+)/ && defined($Distances{"M$1"})) {
            $distance = $Distances{"M$1"};
        }
    }
    $other = spline("OTHER");
    $other =~ s/\s*-\s*/-/g;
    $other =~ s/^( *)M ([0-9])/$1M$2/;
    $descr = line("OBJECT").spline("TYPE").spline("CLASS").$other;
    if ( substr($descr,32) =~ /[;0-9]/ ) {
        if ( $other =~ /\;/ ) {
            $other =~ s/\;.*//;
            $descr = line("OBJECT").spline("TYPE").spline("CLASS").$other;
            if ( substr($descr,32) =~ /[0-9]/) {
                $descr = line("OBJECT").spline("TYPE").spline("CLASS");
            }
        }
        else {
            $descr = line("OBJECT").spline("TYPE").spline("CLASS");
        }
    }
    print OUT $descr."\t";
    print OUT radec(line("RA"))."\t".radec(line("DEC"))."\t";
    if ( 70 <= $m ) {
        print OUT "--\t";
    }
    else {
        print OUT $m."\t";
    }
    $smax = line("SIZE_MAX");
    $smin = line("SIZE_MIN");
    if ( $smin eq "" ) {
        $smin = $smax;
    }
    elsif ( $smax eq "" ) {
        $smax = $smin;
    }
    if ( $smax eq "" ) {
        print OUT "--";
    }
    else {
        if ( $smax =~ /s/ ) {
            print OUT ($smax/60.);
        }
        else {
            print  OUT ($smax + 0.0);
        }
        if ( $smin ne "" ) {
            if ( $smin =~ /s/ ) {
                print OUT "x".($smin/60.);
            }
            else {
                print OUT "x".($smin + 0.0);
            }
        }
    }
    print OUT "\t--\t$distance" if $distance >= 0;
    print OUT "\n";
}
close SAC;
close OUT;

sub radec {
    my $l = shift;
    $l =~ s/ +/\t/g;
    $l =~ s/^\+//;
    return $l;
}

sub GetLine {
    my $l = shift;
    $l =~ s/^\"//;
    $l =~ s/\"$//;
    $l =~ s/ +\"/\"/g;
    return split /\",\"/, $l;
}

sub line {
    my $ind = shift;
    return $line[$I{$ind}];
}

sub spline {
    my $ind = shift;
    if ( $line[$I{$ind}] ne "" ) {
        return " " . $line[$I{$ind}];
    }
    else {
        return "";
    }
}
