use POSIX qw(floor);

open NAMES, "constellations.txt" or die;
while(<NAMES>) {
    s/[\r\n\s]+$//;
    if (/^([A-Z0-9a-z]+)\s+([A-Z0-9a-z\s]+)/) {
         $name{uc($1)}=$2;
    }
}
close NAMES;

$text  = "";
$const = "";
$count = 0;

open CONST, "bound_20.dat" or die;
while(<CONST>) {
    s/[\r\n]//g;
    s/#.*//;
    $last = -1;
    if (/([.0-9]+)\s+([-+])([.0-9]+)\s+([A-Z0-9]+)/) {
        $ra = $1;
        $dec = $3;
        $dec = -$dec if ($2 eq "-");
        $newConst = $4;

        if ($newConst ne $const) {
            if ($count) {
                ScanConstellation();
            }
            $const = $newConst;
            $count = 0;
        }

        ($current_x[$count], $current_y[$count], $current_z[$count]) = GetCoords($ra,$dec);
        $count++;
    }
}
if ($count) {
    ScanConstellation();
}
close CONST;

sub GetCoords {
     my $ra  = shift;
     my $dec = shift;

     my $pi = 3.141592653589793238462643383;

     $ra *= $pi/12.;
     $dec *= $pi/180.;
     my $cosdec = cos($dec);
     return (cos($ra) * $cosdec, sin($ra) * $cosdec, sin($dec));
}

sub sqr {
     my $x = shift;
     return $x * $x;
}

sub Fits {
     my $x = shift;
     my $y = shift;
     my $z = shift;
     my $r = shift;

     my $r2 = $r * $r;
     
     my $maxD2;

     $maxD2 = -1;

     for my $i (0..$count-1) {
         my $p = $current_x[$i]*$x + $current_y[$i]*$y + $current_z[$i]*$z;
         if ( $p <= 0 ) {
             return -1;
         }
         my $d2 = sqr($current_x[$i]-$x*$p) + sqr($current_y[$i]-$y*$p) + sqr($current_z[$i]-$z*$p);
         if ($r2 < $d2) {
             return -1;
         }
         if ($maxD2 < $d2) {
             $maxD2 = $d2;
         }
     }

     die if $maxD2 < 0;

     return sqrt($maxD2);
}

sub ScanConstellation {
     my $i;
     my $cr;
     my $ra;
     my $dec;

     print STDERR "[$const]";

     for ($cr = 0.02 ; $cr <= 1 ; $cr += .02) {
          print STDERR ".";
          for($ra = 0 ; $ra <= 24 ; $ra += 0.05) {
               for($dec = -90 ; $dec <= 90 ; $dec += .5) {
                    my ($x,$y,$z) = GetCoords($ra,$dec);
                    my $r = Fits($x,$y,$z,$cr);
                    if ( 0 <= $r ) {
                         print "$name{$const}\t$x\t$y\t$z\t$r\n";
                         return;
                    }
               }
          }
     }
     die "Cannot find center";
}
