/*
 * Astro Info - a an astronomical calculator/almanac for PalmOS devices.
 * 
 * $Id: DSTDatabase.java 976 2010-04-14 23:17:51Z hoenicke $
 * Copyright (C) 2002, Astro Info SourceForge Project
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package net.sf.astroinfo;
import net.sf.astroinfo.pdb.*;
import java.io.*;
import java.util.*;


public class DSTDatabase implements AstroInfoConstants {
    Database  pdb;
    String    name;
    TreeMap<String, DSTRecord> entries = new TreeMap<String, DSTRecord>();
    static int version = 0;

    public DSTDatabase() {
    }

    public void read(InputStream is) throws IOException {
	Database pdb = new Database();
	try {
	    pdb.read(is, null, null, DSTRecord.class);
	} catch (InstantiationException ex) {
	    throw new InternalError();
	} catch (IllegalAccessException ex) {
	    throw new InternalError();
	}
	this.name = pdb.getName();
	DSTRecord[] recs = (DSTRecord[]) pdb.getEntries();
	entries.clear();
	for (int i = 0; i < recs.length; i++)
	    addEntry(recs[i]);
    }

    public void write(OutputStream os) throws IOException {
	Database pdb = new Database();
	pdb.setName(name);
	pdb.setVersion(version);
	pdb.setAttributes(Database.DMHDRATTRBUNDLE+Database.DMHDRATTRBACKUP);
	pdb.setCreator(creatorID);
	pdb.setType(dstID);
	DSTRecord[] recs = entries.values().toArray(new DSTRecord[0]);
	pdb.setEntries(recs);
	pdb.write(os);
    }

    public void parse(Reader r) throws IOException {
	BufferedReader br = new BufferedReader(r);
	String line = br.readLine();
	name = line;
	while (br.ready()) {
	    line = br.readLine();
	    DSTRecord rec = new DSTRecord();
	    rec.parse(line);
	    addEntry(rec);
	}
    }
    
    public void dump(Writer w) throws IOException {
	PrintWriter pw = new PrintWriter(w);
	pw.println(name);
	for (DSTRecord entry : entries.values()) {
	    pw.println(entry.toString());
	}
	pw.flush();
    }
    
    public DSTRecord getEntry(String name) {
	return entries.get(name);
    }

    public void addEntry(DSTRecord rec) {
	entries.put(rec.getName(), rec);
    }    

    public static void usage() {
	System.err.println("SYNTAX: java net.sf.astroinfo.DSTDatabase"+
			   " [-rv] <infile> <outfile>");
	System.err.println("\t-r:\tfrom binary to text (default is reverse)");
	System.err.println("\t-v <num>:\tset version number)");
    }

    public static void main(String[] args) throws IOException {
	boolean reverse = false;
	int argsOffset = 0;
        while (argsOffset < args.length && args[argsOffset].charAt(0)=='-') {
            switch (args[argsOffset].charAt(1)) {
	    case 'r':
		reverse = true;
		break;
	    case 'v':
		version = Integer.parseInt(args[++argsOffset]);
		break;
	    default:
		System.err.println("Invalid option: "+args[argsOffset]);
		usage();
		return;
            }
            argsOffset++;            
        }

	if (args.length - argsOffset != 2) {
	    usage();
	    return;
	}

	DSTDatabase db = new DSTDatabase();
	if (reverse) {
	    db.read(new FileInputStream(args[argsOffset]));
	    if (args[argsOffset+1].equals("-"))
		db.dump(new OutputStreamWriter(System.out));
	    else
		db.dump(new FileWriter(args[argsOffset+1]));
	} else {
	    if (args[argsOffset].equals("-"))
		db.parse(new InputStreamReader(System.in));
	    else
		db.parse(new FileReader(args[argsOffset]));
	    db.write(new FileOutputStream(args[argsOffset+1]));
	}
    }
}
