/*
 * Astro Info - a an astronomical calculator/almanac for PalmOS devices.
 *
 * $Id: Astro.c 992 2010-06-22 03:39:39Z arpruss $
 * Copyright (C) 2001, Astro Info SourceForge Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <PalmOS.h>
#include "AstroGlue.h"
#include "resource.h"
#include "Astro.h"
#include "AstroLib.h"
#include "AstroCat.h"
#include "AstroDetails.h"
#include "AstroSky.h"
#include "FixedPoint.h"
#include "Grayscale.h"
#include "AstroDST.h"
#include "AstroTime.h"
#include "AstroOptions.h"
#include "AstroSS.h"
#include "AstroPhaseCal.h"
#include "AstroList.h"
#include "AstroFetchImage.h"
#include "AstroAlign.h"
#include "Bitmap.h"
#include "resize.h"

#include "palmprof.h"
// Palm Reporter support
#include <HostControl.h>

// This is the minimum number of icons in the icon menu
#define NUMSTANDARDICONS 6

Char*                   astroInfoArgument = "";
SystemPreferencesType   sysPrefs;
UserPrefsType           userPrefs;

/* Used to hold the screen resolution of the PDA */
RectangleType           screenRect;

static ListPtr         iconmenuPtr;

Char dstRuleName[FIELDLENGTH];

/* These can change depending on the screen resolution of the PDA */
FontID fontSmall    = 130;  // Our custom font.
FontID fontNormal   = stdFont;
FontID fontBold     = boldFont;

UInt16 mainMenu = m_Normal;


UserPrefsType defaultPrefs = {
    ASTROVERSION,             /* version */
    {
        {0,                        /* dstRuleNr */
         0,                        /* tz */
         FP(0),                    /* lat */
         FP(0),                    /* lng */
         { 0, },                   /* dstRule */
	 ""
        },
        {0,                        /* dstRuleNr */
         0,                        /* tz */
         FP(0),                    /* lat */
         FP(0),                    /* lng */
         { 0, },                   /* dstRule */
	 ""
        },
        {0,                        /* dstRuleNr */
         0,                        /* tz */
         FP(0),                    /* lat */
         FP(0),                    /* lng */
         { 0, },                   /* dstRule */
	 ""
        },
        {0,                        /* dstRuleNr */
         0,                        /* tz */
         FP(0),                    /* lat */
         FP(0),                    /* lng */
         { 0, },                   /* dstRule */
	 ""
        },
        {0,                        /* dstRuleNr */
         0,                        /* tz */
         FP(0),                    /* lat */
         FP(0),                    /* lng */
         { 0, },                   /* dstRule */
	 ""
        }
    },
    0, 0,                    /* currentLoc, padding */
    FP_ONE, 0,               /* alt/az */
    f_LocationOptions,       /* default active FormId */
    30,30,1,                   /* Time increment, interval, rocker interval */
    1,                       /* radec2000 */
    0,                       /* parsec */
    0,                       /* slow scroll */
    0,                       /* reserved */
    PAGEBTNROCKER,           /* pageBtnFunction */
    4,                       /* Default number of detail rows */
    1,                       /* Vertical spacing of detail rows. Default is Nice */
    { /* skyFlags */
        0, 0, 0, 0, 2,
        0, /* displayPosText */
        1, /* displayPosGraph */
        1, /* displayZoom */
        2, /* sizePosCircles */
        0, /* noDisplayTimeout */
        0, /* measureMode */
        0  /* showAllNonStars */
    },
    { /* optFlags */
        1, /* time24 */
        1, /* moreAccurate */
        0, /* nightMode */
        1, /* blackBackground */
        0, /* showAbove */
        1, /* showBelow */
        0, /* showNever */
        0, /* timeStopped false */
        1, /* isNow true */
        0, /* fontsize */
        0, /* omit stars */
        ORIENTATION_CORRECT_IMAGE, /* orientation */
        0, /* fadeBelowHorizon */
        0, /* showConstNames */
        0xfff /* reserved as 1 */
    },
    0L, 0L,                 /* julian day/time */
    NO_OBJECT,              /* currentObj */
    24,                     /* Zoom value (all the way out) */
    0,                      /* Detail page */
    1,   /* default format for hours and degrees display */
    0,   /* last constellation */
    0,   /* pad */
    { { { 0, 255, 127,   0 }, /* on black: PLANETS*/
	{ 0, 255, 255,   0 }, /*           MOON   */
	{ 0, 255, 255, 192 }, /*           SUN    */
	{ 0, 192, 192, 192 }, /*           LINES  */
	{ 0, 192, 192, 192 }, /*           EXTRAS */
	{ 0,   0,   0,   0 }, /*           TEXT   */
	{ 0,  63,  63,  63 }, /*           SHADOW */
	{ 0,  255, 0,    0 }, /*           CIRCLES */

	{ 0, 255, 255, 255 }, /*           RESERVED    */
	{ 0, 255, 255, 255 }, /*           RESERVED    */
	{ 0, 255, 255, 255 }, /*           RESERVED    */
	{ 0, 255, 255, 255 }, /*           RESERVED    */
	{ 0, 255, 255, 255 }, /*           RESERVED    */
	{ 0, 255, 255, 255 }, /*           RESERVED    */
	{ 0, 255, 255, 255 }, /*           RESERVED    */
	{ 0, 255, 255, 255 }, /*           RESERVED    */

	},
      { { 0,   0,   0, 255 }, /* on white: PLANETS*/
	{ 0, 255, 192,   0 }, /*           MOON   */
	{ 0, 255, 127,   0 }, /*           SUN    */
	{ 0,   0,   0,   0 }, /*           LINES  */
	{ 0,   0,   0,   0 }, /*           EXTRAS */
	{ 0,   0,   0,   0 }, /*           TEXT   */
	{ 0, 223, 223, 223 }, /*           SHADOW */
	{ 0, 255,  0,    0 }, /*           CIRCLES */

	{ 0,   0,   0,   0 }, /*           RESERVED  */
	{ 0,   0,   0,   0 }, /*           RESERVED  */
	{ 0,   0,   0,   0 }, /*           RESERVED  */
	{ 0,   0,   0,   0 }, /*           RESERVED  */
	{ 0,   0,   0,   0 }, /*           RESERVED  */
	{ 0,   0,   0,   0 }, /*           RESERVED  */
	{ 0,   0,   0,   0 }, /*           RESERVED  */
	{ 0,   0,   0,   0 }, /*           RESERVED  */

	},
      { { 0, 255, 127,   0 }, /* nightmode:PLANETS*/
	{ 0, 255, 255,   0 }, /*           MOON   */
	{ 0, 255, 255, 192 }, /*           SUN    */
	{ 0, 160,   0,   0 }, /*           LINES  */
	{ 0, 160,   0,   0 }, /*           EXTRAS */
	{ 0, 160,   0,   0 }, /*           TEXT   */
	{ 0,  63,  63,  63 }, /*           SHADOW */
	{ 0,  100,  0,   0 }, /*           CIRCLES */

	{ 0, 160,   0,   0 }, /*           RESERVED  */
	{ 0, 160,   0,   0 }, /*           RESERVED  */
	{ 0, 160,   0,   0 }, /*           RESERVED  */
	{ 0, 160,   0,   0 }, /*           RESERVED  */
	{ 0, 160,   0,   0 }, /*           RESERVED  */
	{ 0, 160,   0,   0 }, /*           RESERVED  */
	{ 0, 160,   0,   0 }, /*           RESERVED  */
	{ 0, 160,   0,   0 }, /*           RESERVED  */

      } },
    MODE_ONBLACK,
    0, /* circles */
    { 0,0,0,0,0,0 }, /* custom circle */
    "DSS2 Red",
    { 0 } /* more custom circles */
};

/* Prototypes we don't want to advertise outside of this file */

/******************************************************************************/ 

/* This function loads the user preferences with some version upgrade smarts
 * to preserve previous options where possible.
 *
 * The flag passed determines whether to reset preferences to default(true).
 */
void OPTIONSECTION
GetUserSettings(Boolean reset)
{
    UInt16 mptSize = sizeof(userPrefs);
    UInt16 oldSize;

    HostTraceOutputTL(appErrorClass, "GetUserSettings: reset=%s", (reset?"true":"false"));

    // Fill with defaults.
    MemMove(&userPrefs, &defaultPrefs, sizeof(userPrefs));
    if (kDensityDouble <= screenDensity && userPrefs.zoomValue == 24) {
        userPrefs.zoomValue = 23;
    }

    oldSize = 0;
    if (noPreferenceFound != PrefGetAppPreferences(APPID, 0, NULL, &oldSize, true)) {
        /* Work out how much of the preferences is the same depending on the
         * version of AI. Note check for reset to ignore using any existing
         * preferences if reset is true.
         */
        if (oldSize < mptSize)
            mptSize = oldSize;
        if ((PrefGetAppPreferences(APPID, 0,
                                   &userPrefs,
                                   &mptSize,
                                   true) != noPreferenceFound)
            && userPrefs.version != ASTROVERSION && !reset) {

            Int16 compsize = ASTROCOMPSIZE(userPrefs.version);
            if (compsize == 0) {
                /* We found a very old version, warn user about userPrefs
                 * changes.
                 */
                FrmAlert(a_PrefsChanged);
            }

            MemMove((Char *)&userPrefs + compsize,
                    (Char *)&defaultPrefs + compsize,
                    sizeof(userPrefs) - compsize);

            userPrefs.version = ASTROVERSION;
        }
    }

    /* Avoid hogging the system if user prefs broken */
    if (userPrefs.timeInt <= 0)
        userPrefs.timeInt = 1;

    // Get the current time
    CalculateDate();
    if (userPrefs.optFlags.isNow)
        SetNow();
}

/* These allow for simpler handling code. Will need updating when any
 * significant form changes are made. */
static UInt32 icon_formID[] = {
    0, f_Catalog, f_SolarSystem, f_PhaseCalendar, f_ObjectDetail, f_List
};

static UInt16 icon_imageID[] = {
    bmp_options, bmp_cat, bmp_ss, bmp_phasecal, bmp_sky, bmp_list
};

/*
 * Display the current catalog of objects.
 */
static void DBSECTION
IconListDrawData(Int16 itemNum, RectangleType *bounds, Char **itemsText)
{
    MemHandle handle;
    BitmapPtr pIconBmp;
    Char s[32];
//    UInt32 formID = FrmGetActiveFormID();

    HostTraceOutputTL(appErrorClass, "IconListDrawData %hd", itemNum);

    handle = DmGetResource('Tbmp', icon_imageID[itemNum]);

    pIconBmp = (BitmapPtr)MemHandleLock(handle);
    if (pIconBmp) {
        // Use normal API as we want this to be resolution independent
        WinDrawBitmap(pIconBmp, 0, bounds->topLeft.y);
        MemHandleUnlock(handle);
    }
    DmReleaseResource(handle);

    SysStringByIndex(stbl_IconMenu, (UInt16)itemNum, s, sizeof(s));
    WinDrawChars(s, StrLen(s), bounds->topLeft.x + 11, bounds->topLeft.y);
}


static void ASTROSECTION
SendSkyMapCommand(UInt16 cmd)
{
    EventType event;

    if (userPrefs.activeFormId != f_ObjectDetail &&
        userPrefs.activeFormId != f_FullScreen) {

        FrmGotoForm(f_ObjectDetail);
    }
    MemSet(&event, sizeof(EventType), 0);
    event.eType = cmd;
    EvtAddEventToQueue(&event);
}

/*
 * Process all input and menu selections. This is possible because
 * all non-modal Astro forms use the same menu.
 */
Boolean ASTROSECTION
FormChangeHandleEvent(EventPtr event)
{
    UInt32  formID = FrmGetActiveFormID();
    FormPtr  formPtr = FrmGetActiveForm();
    Boolean handled = false;

    switch (event->eType) {

    case menuOpenEvent:
        /* ideally, we should check which menu is loaded, but
        the rscID field in the event data is broken, so this would
        require careful keeping track.  For now, we assume that this
        function is only called in non-modal forms that have the same
        menu. */
        if(CheckMinRomVersion(sysVersion40) && formID != f_Align) {
            if (NULL==GetWebBrowser()) {
                 MenuHideItem(mi_WebLookup);
            }
            if (NULL==GetDictionary())
                 MenuHideItem(mi_DictLookup);
        }
        break;
    case menuEvent:
        switch (event->data.menu.itemID) {
            case mi_SolarSystem:
                if (userPrefs.activeFormId != f_SolarSystem) {
                    FrmGotoForm(f_SolarSystem);
                    handled = true;
                }
                break;

            case mi_About:
                if (userPrefs.activeFormId != f_About)
                    FrmPopupForm(f_About);
                handled = true;
                break;

            case mi_Options:
                FrmGotoForm(f_GeneralOptions);
                handled = true;
                break;

            case mi_Circles:
                SendSkyMapCommand(chooseCirclesEvent);
                handled = true;
                break;

            case mi_GoToRADec:
                SendSkyMapCommand(gotoRaDecEvent);
                handled = true;
                break;

            case mi_ZoomOut:
                SendSkyMapCommand(zoomOutEvent);
                handled = true;
                break;

            case mi_DSSLookup:
                SendSkyMapCommand(dssLookupEvent);
                handled = true;
                break;

            case mi_GoToConstellation:
                SendSkyMapCommand(gotoConstellationEvent);
                handled = true;
                break;

            case mi_Align:
                SendSkyMapCommand(gotoAlignEvent);
                handled = true;
                break;

            case mi_MeasureMode:
                userPrefs.skyFlags.measureMode =
                    ! userPrefs.skyFlags.measureMode;
                handled = true;
                break;

            case mi_Catalog:
                if (userPrefs.activeFormId != f_Catalog) { 
                    FrmGotoForm(f_Catalog);
                    handled = true;
                }
                break;

            case mi_LunarPhase:
                if (userPrefs.activeFormId != f_PhaseCalendar) {
                    FrmGotoForm(f_PhaseCalendar);
                    handled = true;
                }
                break;

            case mi_AddToList: {
                EventType event;

                if (GetCurrentObject() == NO_OBJECT) {
                    FrmAlert(a_NoSelectedObject);
                }
                else {
                    if (userPrefs.activeFormId != f_List) {
                        FrmGotoForm(f_List);
                    }
                    MemSet(&event, sizeof(EventType), 0);
                    event.eType = listAddEvent;
                    EvtAddEventToQueue(&event);
                    handled = true;
                }    
                break;
            }

            case mi_Timebases:
                if (userPrefs.activeFormId != f_Timebases) {
                    FrmGotoForm(f_Timebases);
                    handled = true;
                }
                break;

            case mi_WebLookup:
                if (userPrefs.activeFormId != f_List)
                    LookupObject(GetCurrentObject(), NULL, LOOKUP_WEB);
                else
                    ListLookupItem(LOOKUP_WEB);
                handled = true;
                break;

            case mi_DictLookup:
                if (userPrefs.activeFormId != f_List)
                    LookupObject(GetCurrentObject(), NULL, LOOKUP_DICT);
                else
                    ListLookupItem(LOOKUP_DICT);
                handled = true;
                break;

            case mi_Sky:
                if (userPrefs.activeFormId != f_ObjectDetail) {
                    FrmGotoForm(f_ObjectDetail);
                    handled = true;
                }
                break;
                
            case mi_Help:
                switch (userPrefs.activeFormId)
                {
                    case f_ObjectDetail:
                    case f_FullScreen:
                        FrmHelp(s_DetailHelp);
                        break;
                    case f_Timebases:
                        FrmHelp(s_TimebasesHelp);
                        break;
                    case f_PhaseCalendar:
                        FrmHelp(s_PhaseCalendarHelp);
                        break;
                    case f_List:
                        FrmHelp(s_ListHelp);
                        break;
                    case f_SkyOptions:
                        FrmHelp(s_SkyOptionsHelp);
                        break;
                    case f_ColorOptions:
                        FrmHelp(s_ColorOptionsHelp);
                        break;
                    case f_GeneralOptions:
                        FrmHelp(s_GeneralOptionsHelp);
                        break;
                    case f_LocationOptions:
                        FrmHelp(s_LocationOptionsHelp);
                        break;
                    case f_CatalogOptions:
                        FrmHelp(s_CatalogOptionsHelp);
                        break;
                    case f_SolarSystem:
                        FrmHelp(s_SolarSystemHelp);
                        break;
                    case f_Catalog:
                        FrmHelp(s_CatalogHelp);
                        break;
                    default:
                        ErrFatalDisplay("Unknown form. Missing Help!");
                }
                handled = true;
                break;
    
            case mi_Night: {
                ColorModeType curMode;

                curMode  = GetColorMode();

                /* Color support is enabled. Toggle night mode. */
                if (userPrefs.optFlags.nightMode) {
                    if (userPrefs.prevColorMode == MODE_ONBLACK)
                        userPrefs.optFlags.blackBackground = true;
                    else
                        userPrefs.optFlags.blackBackground = true;

                    userPrefs.optFlags.nightMode = false;
                    userPrefs.prevColorMode = curMode;
                }
                else {
                    userPrefs.optFlags.nightMode = true;
                    userPrefs.optFlags.blackBackground = true;
                    userPrefs.prevColorMode = curMode;
                }
                NightMode();
                FrmUpdateForm(formID, frmRedrawUpdateCode);
                handled = true;
                break;
            }
        }
        break;

        case ctlSelectEvent:
            switch (event->data.ctlSelect.controlID) {
                
                case b_FormMenu:
                    HostTraceOutputTL(appErrorClass, "b_FormMenu");
                    {
                        Int16 opt;
                        
                        opt = LstPopupList((ListPtr)FrmGetObjectPtrByID(formPtr, l_IconMenu));

                        // We skip option 0 because it is form sensitive and handled below
                        if (opt > 0 && opt < NUMSTANDARDICONS) {
			                FrmGotoForm(icon_formID[opt]);
                        } else if (opt == 0) {
                            // Options
                            // Could also be -1 if no list item is selected
                            switch (formID) {

                            case f_List:
                            case f_SolarSystem:
                                FrmGotoForm(f_GeneralOptions);
                                break;
                                    
                            case f_PhaseCalendar:
                            case f_ObjectDetail:
                                FrmGotoForm(f_SkyOptions);
                                break;
                                
                            case f_Catalog:
                                FrmGotoForm(f_CatalogOptions);
                                break;
                                
                            default:
                            }
                        }
                    }
                    handled = true;
                    break;
            }
            break;
        
        default:
    }
    return handled;
}



static Boolean ASTROSECTION
AboutFormHandleEvent(EventPtr event)
{
    FormPtr formPtr = FrmGetActiveForm();
    Boolean handled = false;
    static  MemHandle versionHandle, contentHandle;

    switch (event->eType) {
    case ctlSelectEvent:
        if (event->data.ctlSelect.controlID == b_OK) {

            FldSetTextHandle((FieldPtr)FrmGetObjectPtrByID(formPtr, f_Version),
                             NULL);
            FldSetTextHandle((FieldPtr)FrmGetObjectPtrByID(formPtr, f_Content),
                             NULL);
            DmReleaseResource(versionHandle);
            DmReleaseResource(contentHandle);
            
            FrmReturnToForm(0);
            handled = true;
            break;
        }
        break;

    case frmCloseEvent:
        FldSetTextHandle((FieldPtr)FrmGetObjectPtrByID(formPtr, f_Version),
                         NULL);
        FldSetTextHandle((FieldPtr)FrmGetObjectPtrByID(formPtr, f_Content),
                         NULL);
        DmReleaseResource(versionHandle);
        DmReleaseResource(contentHandle);
        break;

    case frmOpenEvent:
        versionHandle = DmGetResource('tver', 1);
        contentHandle = DmGetResource('tSTR', s_About);
        FldSetTextHandle((FieldPtr)FrmGetObjectPtrByID(formPtr, f_Version),
                         versionHandle);
        
        FldSetTextHandle((FieldPtr)FrmGetObjectPtrByID(formPtr, f_Content),
                         contentHandle);

        SetMenu(formPtr);
        FrmDrawForm(formPtr);
        
        handled = true;
        break;
        
    default:
    }
    return handled;
}



/*
 * Events for the main application (basically, setting form event
 * handlers.)
 */
static Boolean ASTROSECTION
ApplicationHandleEvent(EventPtr event)
{
    FormPtr     formPtr;
    Boolean     handled = false;
    Int16       iconMenuEntry = -1;
    UInt32      formID = FrmGetActiveFormID();
    static      Coord lastExtentX = -1;
    static      Coord lastExtentY = -1;
    Coord       extentX;
    Coord       extentY;

    switch (event->eType) {
        case winDisplayChangedEvent:
            if(FrmGetActiveForm()==NULL)
                break;
            GetScreenDimensions(FrmGetActiveForm(), &screenRect);
            WinGetDisplayExtent( &extentX, &extentY );
            if ( extentX == lastExtentX && extentY == lastExtentY )
                return true;
            lastExtentX = extentX;
            lastExtentY = extentY;
            break;
        case frmOpenEvent:
            WinGetDisplayExtent( &lastExtentX, &lastExtentY );
            GetScreenDimensions(FrmGetActiveForm(), &screenRect);
            break;
        case  frmLoadEvent:
            WinGetDisplayExtent( &lastExtentX, &lastExtentY );
            formID = event->data.frmLoad.formID;
            formPtr = FrmInitForm(formID);
            FrmSetActiveForm(formPtr);
            SetResizePolicy(formID);
            /* Use this for scaling display
             * the form is 2 pixel bigger than the screen.
             */

            GetScreenDimensions(formPtr, &screenRect);
            
            switch (formID) {
                case f_ChooseCircles:
                    FrmSetEventHandler(formPtr,
                                       ChooseCirclesFormHandleEvent);
                    handled = true;
                    break;

                case f_SolarSystem:
                    userPrefs.activeFormId = formID;
                    FrmSetEventHandler(formPtr,
                                       SolarSystemFormHandleEvent);
                    iconMenuEntry = 2;
                    handled = true;
                    break;
    
                case f_DSSImage:
                    FrmSetEventHandler(formPtr,
                                       DSSImageFormHandleEvent);
                    handled = true;
                    break;

                case f_ObjectDetail:
                case f_FullScreen:
                    userPrefs.activeFormId = formID;
                    FrmSetEventHandler(formPtr,
                                       SkyFormHandleEvent);
                    if (formID != f_FullScreen)
                        iconMenuEntry = 4;
                    handled = true;
                    break;

                case f_List:
                    userPrefs.activeFormId = formID;
                    FrmSetEventHandler(formPtr,
                                       ListFormHandleEvent);
                    iconMenuEntry = 5;
                    handled = true;
                    break;

                case f_NoteEdit:
                    FrmSetEventHandler(formPtr,
                                       NoteEditFormHandleEvent);
                    handled = true;
                    break;

                case f_PhaseCalendar:
                    userPrefs.activeFormId = formID;
                    FrmSetEventHandler(formPtr,
                                       PhaseCalendarFormHandleEvent);
                    iconMenuEntry = 3;
                    handled = true;
                    break;

                case f_Align:
                    FrmSetEventHandler(formPtr,
                                       AlignFormHandleEvent);
                    handled = true;
                    break;

                case f_Catalog:
                    userPrefs.activeFormId = formID;
                    FrmSetEventHandler(formPtr,
                                       CatalogFormHandleEvent);
		            iconMenuEntry = 1;
                    handled = true;
                    break;
                    
                case f_SelectCatalog:
                    FrmSetEventHandler(formPtr,
                                       CatalogListFormHandleEvent);
                    handled = true;
                    break;
    
                case f_Timebases:
                    userPrefs.activeFormId = formID;
                    FrmSetEventHandler(formPtr,
                                       TimebasesFormHandleEvent);
		    iconMenuEntry = 0;
                    handled = true;
                    break;
    /**********/
                case f_LocationOptions:
                case f_GeneralOptions:
                case f_CatalogOptions:
                case f_SkyOptions:
                case f_ColorOptions:
                    userPrefs.activeFormId = formID;
                    FrmSetEventHandler(formPtr,
                                       AllOptionsFormHandleEvent);
                    iconMenuEntry = 0;
                    handled = true;
                    break;

                case f_About:
                    FrmSetEventHandler(formPtr,
                                       AboutFormHandleEvent);
                    handled = true;
                    break;
            }
            
            if (iconMenuEntry >= 0) {
                iconmenuPtr = (ListPtr)FrmGetObjectPtrByID(formPtr, l_IconMenu);
                LstSetDrawFunction(iconmenuPtr, IconListDrawData);
                LstSetListChoices(iconmenuPtr, NULL, NUMSTANDARDICONS);
		LstSetSelection(iconmenuPtr, iconMenuEntry);
            }
            break;
            
        default:
    }

    return handled;
}



#ifdef SELFTEST
extern void ReportTest(Char* text);



void
ReportTest(Char* text)
{
    FrmCustomAlert(a_Warning, text, " ", " ");
}



static void RunTests() {
    DSTTest();
    LibTest();
}
#endif



/*
 * Initialise our custom fonts; allocate the handles for
 * various text fields; load our previously saved preferences.
 */
static void
StartApplication(void)
{
    MemHandle fontHandle;
    FontType *fontPtr;

    HostTraceInit();

    HostTraceOutputTL(appErrorClass, "*** StartApplication: Astro Info");

    EnableGraphicsMode();
    if (winVersion >= 4) {
        fontHandle=DmGetResource('nfnt',fnt_AI);
        fontPtr=MemHandleLock(fontHandle);
        FntDefineFont(fontSmall, fontPtr);
    } else {
        fontHandle=DmGetResource('NFNT',fnt_AI);
        fontPtr=MemHandleLock(fontHandle);
        FntDefineFont(fontSmall, fontPtr);
    }

    fontHandle=DmGetResource('NFNT',fnt_Tiny);
    fontPtr=MemHandleLock(fontHandle);
    FntDefineFont(131, fontPtr);

    HostTraceOutputTL(appErrorClass, "PrefGetPreferences");

    PrefGetPreferences(&sysPrefs);

    InitializeResizeSupport( w_Resize );
    LoadResizePrefs( APPID, 10 );

    GetUserSettings(false);

    switch(*astroInfoArgument) {
        case 'f': /* find */
            userPrefs.activeFormId = f_Catalog;
            break;
        case 'c': /* constellation */
            if ( userPrefs.activeFormId != f_FullScreen )
                userPrefs.activeFormId = f_ObjectDetail;
            break;
        default:
            break;
    }

    /* If previous last form doesn't exist in this version then fall back
     * to default form.
     */

    switch (userPrefs.activeFormId) {

    case f_SolarSystem:
    case f_PhaseCalendar:
    case f_Catalog:
    case f_ObjectDetail:
    case f_FullScreen:
    case f_Timebases:
    case f_LocationOptions:
    case f_GeneralOptions:
    case f_CatalogOptions:
    case f_SkyOptions:
    case f_ColorOptions:
    case f_List:
        break;

    default:
        userPrefs.activeFormId = defaultPrefs.activeFormId;
    }

    if (colorSupport) {
        NightMode();
        mainMenu=m_Color;
    }

#ifdef SELFTEST
    RunTests();
#endif

    AlignStart();

    /* Open previously opened catalog.
     * Also initialises catalog for sky map.
     */
    OpenCatalogs();

    HostTraceOutputTL(appErrorClass, "Opening previous form %hu", userPrefs.activeFormId);

    FrmGotoForm(userPrefs.activeFormId);

    if ( astroInfoArgument[0] == 'c' ) {
        SendSkyMapCommand(findConstellationEvent);
    }
}



/*
 * Releases all memory; closes libraries; saves preferences
 */
static void
StopApplication()
{
    MemHandle fontHandle;

    NetClose();

    NightModeCleanup();

    FreeCatalogs();
    AlignClose();

    SaveResizePrefs( APPID, 10, 0 );
    TerminateResizeSupport();

    FrmCloseAllForms();

    fontHandle = DmGetResource(winVersion >= 4 ? 'nfnt' : 'NFNT', fnt_AI);
    MemHandleUnlock(fontHandle);

    fontHandle = DmGetResource('NFNT',fnt_Tiny);
    MemHandleUnlock(fontHandle);

    PrefSetAppPreferences(APPID,
                          0, 1,
                          &userPrefs,
                          sizeof(userPrefs),
                          true);

    DisableGraphicsMode();

    HostTraceOutputTL(appErrorClass, "StopApplication: Astro Info");

    HostTraceClose();
}



/*
 * Application event loop. duh.
 */
static ASTROSECTION void
EventLoop(void)
{
    EventType   event;
    UInt16      error;
    Int32       timeout = evtWaitForever;

    do {
        if (!CheckMinRomVersion(sysVersion35)) {
            timeout = GlueNullTick - TimGetTicks();
            if (timeout < 0)
                timeout = 0;
        }
        EvtGetEvent(&event, timeout);
        
        if (!SysHandleEvent(&event)
            && !MenuHandleEvent(NULL, &event, &error)
            && !ResizeHandleEvent(&event)
            && !ApplicationHandleEvent(&event))
            FrmDispatchEvent(&event);
    } while (event.eType != appStopEvent);
}


void
SetMenu(FormPtr formPtr)
{
    FrmSetMenu(formPtr, mainMenu);
}


UInt32
PilotMain(UInt16 cmd,
          MemPtr cmdBPD,
          UInt16 launchFlags)
{
    if (cmd == sysAppLaunchCmdNormalLaunch)
    {

        FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
        /* Get and store ticks per second */
        ticksPerSec = SysTicksPerSecond();

        if (!CheckMinRomVersion(sysVersion30)) {
            FrmAlert(a_TooLimited);
            return 0;
        }

        if (cmdBPD != NULL)
            astroInfoArgument = cmdBPD;

        /* Main application */
        PROFILER_MAX(100);
        StartApplication();
        EventLoop();
        StopApplication();
        /* End of main application. */

        return 0;
    }
    else if (cmd == sysAppLaunchCmdNotify) {
        if ( launchFlags & sysAppLaunchFlagSubCall )
            HandleResizeNotification( ( (SysNotifyParamType *)cmdBPD )->notifyType );
    }

    return 0;
}
