/*
 * Astro Info - a an astronomical calculator/almanac for PalmOS devices.
 *
 * $Id: AstroCat.h 976 2010-04-14 23:17:51Z hoenicke $
 * Copyright (C) 2001, Astro Info SourceForge Project
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef AstroCat_h
#define AstroCat_h

#include <PalmOS.h>
#include "Astro.h"

#define DBSECTION __attribute__ ((section ("code2")))

#define LINES_DBID 'line'
#define INVALID_LINE_INDEX 0xFFFF
#define ENTRIES_PER_INDEX ((64720 / 6) * 2)

typedef struct {
    UInt32 numStars;
    UInt16 numIndexNodes;
    UInt16 numBWNodes;
    UInt16 numQTreeNodes;
    UInt16 qTreeObjectSize;
    UInt16 maxQTreeNodeSize;
    UInt16 entriesPerIndex;
    UInt8  bytesPerEntry;
    UInt8  distance:1;
    UInt8  isStars:1;
    UInt8  padding:6;
    UInt8  magHigh;
} CatalogAppInfo;

typedef struct {
    UInt16 maxQTreeNodeSize;
    UInt16 qTreeObjectSize;
    UInt8  magHigh;
    UInt8  distance:1;
    UInt8  padding:7;
} CatalogIndexV3Extras;

typedef struct {
    Int16    selectedCatalog;
    UInt16   selectedIndex;
    UInt32   visible[(MAX_DBS+31)/32];
    Char     known[MAX_DBS][32];
    RGBColorType color[NUM_COLOR_MODES][MAX_DBS];
    Char     filterString[FIELDLENGTH];
} CatalogPrefType;

Boolean CatObjDetailFormHandleEvent(EventPtr event) DBSECTION;
Boolean CatalogListFormHandleEvent(EventPtr event) DBSECTION;
Boolean CatalogFormHandleEvent(EventPtr event) DBSECTION;
Boolean ColorTableFormHandleEvent(EventPtr event) DBSECTION;
MemHandle CatalogGetHandle(DBRecord* db, UInt16 index) DBSECTION;
void CatalogFreeHandle(DBRecord* db, MemHandle h) DBSECTION;

void OpenCatalogs(void) DBSECTION;
void FreeCatalogs(void) DBSECTION;
void LoadObject(FormPtr formPtr) DBSECTION;

UInt32 GetTotalRecords(void) DBSECTION;
Boolean GetCatalogObject(UInt32 num, AstroRecordType *art, UInt16* dbNumberP)
     DBSECTION;
UInt32 CatalogGetIndexFromQTree(DBRecord *db, UInt32 qtreeIndex) DBSECTION;

extern Char dbPrefix[];
#endif

