/* $Id: AstroFetchImage.c 971 2010-04-12 22:38:03Z hoenicke $ */
#define ALLOW_ACCESS_TO_INTERNALS_OF_BITMAPS
#include <PalmOS.h>
#include <BmpGlue.h>
#include <NetMgr.h>
#include <VFSMgr.h>
#include <Unix/sys_socket.h>
#include "Astro.h"
#include "AstroSky.h"
#include "AstroFetchImage.h"
#include "AstroLib.h"
#include "Grayscale.h"
#include "resource.h"

Char dssCmd[] = "/cgi-bin/images?Survey=%s&position=%s,%s&Rotation=%s&Size=%s,%s&Pixels=%u,%u&"
                "Projection=Sin&Sampler=%s&Return=BMP";

#define DATA_BUFFER_SIZE    8192
#define TIMEOUT              120

#define BITMAP_ROWS            8

#ifndef SHUT_RDWR
# define SHUT_RDWR netSocketDirBoth
#endif

Int32 dssRA;
Int32 dssDec;
Int32 dssXSize;
Int32 dssYSize;
Int32 dssRotate;
Boolean dssXFlip;


UInt16 errno;

static NetSocketRef controlSocket = -1;

Boolean             netOpen = false;
static Int32 oldTimeout = -1;

static void DSSSECTION
DSSMessage( UInt16 id )
{
    Char text[FIELDLENGTH];
    UInt16 oldCoordSys;

    oldCoordSys = WinGlueSetCoordinateSystem(kCoordinatesStandard);
    GetStringResource(id, text);
    UpdateTextField(f_Message, text);
    FldDrawField(GetObjectPtr(f_Message));
    WinGlueSetCoordinateSystem(oldCoordSys);
}

static void DSSSECTION
HTTPClose( void )
{
    if ( 0 <= controlSocket ) {
        shutdown( controlSocket, SHUT_RDWR );
        close( controlSocket );
        controlSocket = -1;
    }
}


static void DSSSECTION
NetCloseGeneric( Boolean immediate )
{
    if ( 0 <= oldTimeout ) {
        AppNetTimeout = oldTimeout;
        oldTimeout = -1;
    }

    if ( netOpen ) {
        NetLibClose( AppNetRefnum, false ); //non-immediate close
        netOpen = false;
    }
}



void DSSSECTION
NetClose( void )
{
    NetCloseGeneric( false );
}



static Boolean DSSSECTION
BMPShow( Coord x0, Coord y0, Boolean xFlip )
{
    UInt8   *buffer;
    UInt16  i;
    UInt16  inBuffer;
    UInt8*  bmp;
    UInt32  offset;
    Coord   width;
    Coord   height;
    Coord   x;
    Coord   y;
    UInt16  depth;
    Coord   dx;
    UInt8   outDepth;
    Err     error;
    BitmapPtr tmpBitmap = NULL;
    BitmapPtr bitmap = NULL;
    UInt16* grays = NULL;
    Boolean success;
    UInt16    rowBytes;
    UInt16    inRowExtraBytes;

    success = false;

    if ( controlSocket < 0 )
        return false;

    buffer = MemPtrNew(DATA_BUFFER_SIZE);

    if ( buffer == NULL )
        return false;

    DSSMessage(s_MsgWait);

    inBuffer = NetUReadN( controlSocket, buffer, 13);

    if(inBuffer < 13 || StrNCompare(buffer+9, "200 ", 4)) {
        goto END_DRAW;
    }

    DSSMessage(s_MsgFetch);

    inBuffer += NetUReadN(controlSocket, buffer+13, DATA_BUFFER_SIZE-13);
    if(inBuffer < 13+54) {
        goto END_DRAW;
    }

    /* This code will not work if the http+bmp headers are more than
       DATA_BUFFER_SIZE bytes altogether */
    bmp = NULL;
    for (i = 0; i+3+54<inBuffer; i++) {
        if(buffer[i] == '\n' && buffer[i+1] == '\n') {
            bmp = buffer+i+2;
            break;
        }
        else if(buffer[i] == '\r' && buffer[i+1]=='\n' &&
                buffer[i+2] == '\r' && buffer[i+3]=='\n') {
            bmp = buffer+i+4;
            break;
        }
    }

    if (bmp == NULL || bmp[0] != 'B' || bmp[1] != 'M') {
        goto END_DRAW;
    }

    offset = bmp[10] + ((UInt32)bmp[11]<<8) + ((UInt32)bmp[12]<<16) + ((UInt32)bmp[13]<<24);

    if (buffer+inBuffer <= bmp+offset) {
        goto END_DRAW;
    }

    width  = bmp[18] + 256 * bmp[19];
    height = bmp[22] + 256 * bmp[23];
    depth  = bmp[28] + 256 * bmp[29];
    if (width%4 == 0)
        inRowExtraBytes = 0;
    else
        inRowExtraBytes = 4 - (width%4);

    if (depth != 8) {
        goto END_DRAW;
    }

    bmp += offset;
    inBuffer = inBuffer - (bmp-buffer);

    if (screenDepth == 16)
        outDepth = 16;
    else
        outDepth = 4;

    if(winVersion >= 4) {
        tmpBitmap = BmpCreate(width, BITMAP_ROWS, outDepth, NULL, &error);
        if (tmpBitmap == NULL)
            goto END_DRAW;
        bitmap = (BitmapPtr) BmpCreateBitmapV3(tmpBitmap, screenDensity,
                                               BmpGetBits(tmpBitmap),
                                               NULL);
    }
    else {
        bitmap = BmpCreate(width, BITMAP_ROWS, outDepth, NULL, &error);
    }

    if (bitmap == NULL)
        goto END_DRAW;

    BmpGlueGetDimensions(bitmap, NULL, NULL, &rowBytes);

    if (outDepth == 16) {
        UInt16    i;

        grays = MemPtrNew(256*sizeof(UInt16));

        if (grays==NULL) {
            goto END_DRAW;
        }

        for (i=0; i<256; i++) {
            grays[i] = (i>>3)|((i>>2)<<5)|((i>>3)<<11);
        }
    }

    if (xFlip) {
        dx = -1;
    }
    else {
        dx = 1;
    }
    
    for(y=0; y<height; y++) {
        void* out;

        out = (UInt8*)BmpGetBits(bitmap) +
                  (UInt32)(BITMAP_ROWS-1-y%BITMAP_ROWS)*rowBytes;

        if (outDepth == 16) {
            UInt16* out16;

            if (xFlip)
                out16 = (UInt16*)out + width - 1;
            else
                out16 = out;

            for(x=0; x<width; x++) {
                if(!inBuffer) {
                    inBuffer = NetUReadN(controlSocket, buffer, DATA_BUFFER_SIZE);
                    if(inBuffer == 0)
                        goto END_DRAW;
                    bmp = buffer;
                }

                *out16 = grays[*bmp++];
                out16 += dx;
                inBuffer--;
            }
            for (x=0; x<inRowExtraBytes; x++) {
                if(!inBuffer) {
                    inBuffer = NetUReadN(controlSocket, buffer, DATA_BUFFER_SIZE);
                    if(inBuffer == 0)
                        goto END_DRAW;
                    bmp = buffer;
                }
                inBuffer--;
                bmp++;
            }
        }
        else {
            /* This code may lose one vertical column in xFlip mode if the width
               is odd.  I think this is not a big deal. */

            UInt8* out8;

            if (0 == y%BITMAP_ROWS) {
                MemSet(BmpGetBits(bitmap), (UInt32)rowBytes * BITMAP_ROWS, 0);
            }

            if (xFlip)
                out8 = (UInt8*)out + ( width + 1 ) / 2 - 1;
            else
                out8 = out;

            for(x=0; x<width; x++) {
                if(!inBuffer) {
                    inBuffer = NetUReadN( controlSocket, buffer, DATA_BUFFER_SIZE);
                    if(inBuffer == 0)
                        goto END_DRAW;
                    bmp = buffer;
                }

                if (!xFlip) {
                    if (x & 1) {
                        *out8   |= ((255-*bmp++)>>4);
                        out8++;
                    }
                    else {
                        *out8   |= ((255-*bmp++)>>4)<<4;
                    }
                }
                else {
                    if (x & 1) {
                        *out8   |= ((255-*bmp++)>>4)<<4;
                        out8--;
                    }
                    else {
                        *out8   |= ((255-*bmp++)>>4);
                    }
                }

                inBuffer--;
            }
            for (x=0; x<inRowExtraBytes; x++) {
                if(!inBuffer) {
                    inBuffer = NetUReadN(controlSocket, buffer, DATA_BUFFER_SIZE);
                    if(inBuffer == 0)
                        goto END_DRAW;
                    bmp = buffer;
                }
                inBuffer--;
                bmp++;
            }
        }

        if (0 == (y+1)%BITMAP_ROWS && BITMAP_ROWS <= y+1) {
            WinDrawBitmap(bitmap, x0, y0+height-(y+1));
        }
    }

    if (0 != y % BITMAP_ROWS) {
        RectangleType rect;

        rect.topLeft.x = x0;
        rect.topLeft.y = y0;
        rect.extent.x  = width;
        rect.extent.y  = height;

        WinSetClip(&rect);
        WinDrawBitmap(bitmap, x0, y0-(BITMAP_ROWS-(y % BITMAP_ROWS)));
        WinResetClip();
    }
    success = true;

END_DRAW:
    if (bitmap)
        BmpDelete(bitmap);
    if (tmpBitmap)
        BmpDelete(tmpBitmap);
    if (grays)
        MemPtrFree(grays);
    MemPtrFree(buffer);

    return success;
}



static Boolean DSSSECTION
NetOpen( void )
{
    Err    err;
    UInt16 returnValue;

    DSSMessage(s_MsgGetNetwork);

    NetClose();

    errno = 0;

    err = SysLibFind( "Net.lib", &AppNetRefnum );

    if ( err != errNone ) {
        return false;
    }

    err = NetLibOpen( AppNetRefnum, &returnValue );

    if ( err != netErrAlreadyOpen && err != errNone ) {
        return false;
    }

    oldTimeout = AppNetTimeout;

    netOpen    = true;

    return true;
}



static void DSSSECTION
Linger( NetSocketRef socket, Boolean state, UInt16 time )
{
    struct linger l;

    l.l_onoff = state;
    l.l_linger  = time;

    setsockopt( socket, SOL_SOCKET, SO_LINGER, &l, sizeof( l ) );
}



static Boolean DSSSECTION
FetchBMP(Char* hostName, UInt16 port, Char* request, Coord x, Coord y, Boolean xFlip)
{
    Boolean success;
    Char*  command;

    if ( ! netOpen )
        return false;

    DSSMessage(s_MsgConnect);
    if ( port == 0 )
         controlSocket = NetUTCPOpen( hostName, "http", 0 );
    else
         controlSocket = NetUTCPOpen( hostName, NULL, port );

    if ( controlSocket < 0 ) {
        return false;
    }

    Linger( controlSocket, false /* true */, 0 );

    AppNetTimeout = TIMEOUT * SysTicksPerSecond();

    command = MemPtrNew(StrLen(request)+80);
    if (command == NULL) {
        HTTPClose();
        return false;
    }

    DSSMessage(s_MsgRequest);
    StrPrintF(command, "GET %s HTTP/1.0\r\nHost: %s\r\n\r\n", request, hostName);

    if (NetUWriteN( controlSocket, command, StrLen( command ) ) < StrLen( command )) {
        MemPtrFree(command);
        HTTPClose();
        return false;
    }

    MemPtrFree(command);

    success = BMPShow(x, y, xFlip);
    HTTPClose();

    return success;
}


static UInt32 DSSSECTION
Deg10000(Int32 angle)
{
    Int16 d;
    Int16 m;
    Int16 s;

    if(angle<=0)
        return 0;

    AngToDMS(angle, &d, &m, &s);
    
    return d * 10000ul + 10000ul * (m * 60ul + s) / 3600ul;
}


static Boolean DSSSECTION
DSSImage(void)
{
    RectangleType rect;
    Char*         cmd;
    Boolean       success;
    Char          survey[FIELDLENGTH];
    UInt16        i;
    Char          ra[12];
    Char          dec[12];
    Char          xSize[12];
    Char          ySize[12];
    Char          rotate[12];
    Char*         sampler;
    UInt32        xPixSize;
    UInt32        yPixSize;
    Int32         xSize10000;
    Int32         ySize10000;
    Int32         rotate10000;

    Int16         dh;
    Int16         m;
    Int16         s;
    Char          sign;

    AngToHMS(dssRA, &dh, &m, &s);
    StrPrintF(ra, "%02d+%02d+%02d", dh, m, s);

    if(dssDec<0) {
        AngToDMS(-dssDec, &dh, &m, &s);
        sign = '-';
    }
    else {
        AngToDMS(dssDec, &dh, &m, &s);
        sign = '+';
    }
    StrPrintF(dec, "%c%02d+%02d+%02d", sign, dh, m, s);

    xSize10000 = Deg10000(dssXSize);
    if (xSize10000 == 0)
        return false;
    StrPrintF(xSize, "%lu.%04lu", xSize10000 / 10000ul, xSize10000 % 10000ul);
    ySize10000 = Deg10000(dssYSize);
    if (ySize10000 == 0)
        return false;
    StrPrintF(ySize, "%lu.%04lu", ySize10000 / 10000ul, ySize10000 % 10000ul);

    if (dssRotate<0) {
        rotate10000 = Deg10000(-dssRotate);
        StrPrintF(rotate, "-%lu.%04lu", rotate10000 / 10000ul, rotate10000 % 10000ul);
    }
    else {
        rotate10000 = Deg10000(dssRotate);
        StrPrintF(rotate, "%lu.%04lu", rotate10000 / 10000ul, rotate10000 % 10000ul);
    }
    FrmGetObjectBounds(FrmGetActiveForm(), GetObjectIndex(g_Image), &rect);

    WinGlueSetCoordinateSystem(kCoordinatesNative);
    if (winVersion >= 4) {
        WinScaleRectangle(&rect);
    }

    WinEraseRectangle(&rect, 0);

    yPixSize = rect.extent.x * ySize10000 / xSize10000;

    if(yPixSize <= rect.extent.y) {
        xPixSize = rect.extent.x;
    }
    else {
        xPixSize = rect.extent.y * xSize10000 / ySize10000;
        yPixSize = rect.extent.y;
    }

    for(i=0; userPrefs.survey[i]; i++) {
        if(userPrefs.survey[i] == ' ') {
            survey[i] = '+';
        }
        else {
            survey[i] = userPrefs.survey[i];
        }
    }

    survey[i] = 0;
    
    if (dssXSize > DMS_TO_ANG(1,0,0) || dssYSize > DMS_TO_ANG(1,0,0))
        sampler = "NN";
    else
        sampler = "Clip";

    cmd = MemPtrNew(StrLen(dssCmd) + StrLen(survey) + StrLen(ra) +
              StrLen(dec) + StrLen(xSize) + StrLen(ySize) + StrLen(rotate) +
              StrLen(sampler) +
              5 + 5 + 1);
    success = false;

    if(cmd != NULL) {
        Boolean open;
        UInt16  prevCoordSys;

        StrPrintF(cmd, dssCmd, survey, ra, dec, rotate, xSize, ySize,
            (UInt16)xPixSize, (UInt16)yPixSize, sampler);

        prevCoordSys = WinGlueSetCoordinateSystem(kCoordinatesStandard);
        open = NetOpen();
        WinGlueSetCoordinateSystem(prevCoordSys);

        if (open)
            success = FetchBMP("skyview.gsfc.nasa.gov", 80,
                cmd,
                rect.topLeft.x + (rect.extent.x-xPixSize)/2,
                rect.topLeft.y + (rect.extent.y-yPixSize)/2,
                dssXFlip);

        MemPtrFree(cmd);
    }

    WinGlueSetCoordinateSystem(kCoordinatesStandard);

    DSSMessage(s_MsgClose);
    NetClose();
    return success;
}



static void DSSSECTION
DSSDraw(void)
{
    Show(lbl_Error, false);
    Show(lbl_Error, !DSSImage());
    DSSMessage(s_MsgNone);
}


static void DSSSECTION
DSSImageOpenForm(void)
{
    UInt16 i;
    UInt16 n;
    ListType* list;

    if (userPrefs.survey[0] == 0) {
        StrCopy(userPrefs.survey, "DSS2 Red");
    }

    list = GetObjectPtr(l_Survey);

    n = LstGetNumberOfItems(list);
    for(i=0; i<n; i++)
        if(!StrCompare(LstGetSelectionText(list, i), userPrefs.survey))
             break;

    if (i==n) {
        i = 0;
        StrCopy(userPrefs.survey, LstGetSelectionText(list, 0));
    }
    
    LstSetSelection(list, i);
    CtlSetLabel(GetObjectPtr(tr_Survey), userPrefs.survey);

    FrmDrawForm(FrmGetActiveForm());
    DSSDraw();
}



/*
 * Events for the object list
 */
Boolean DSSSECTION
DSSImageFormHandleEvent(EventPtr event)
{
    Boolean handled = false;
    FormPtr formPtr = FrmGetActiveForm();

    switch (event->eType) {
        case ctlSelectEvent: {
            switch (event->data.ctlSelect.controlID) {
                case b_OK:
                    FrmReturnToForm(0);
                    handled = true;
                    break;
            }
            break;
        }

        case frmOpenEvent:
            DSSImageOpenForm();
            handled = true;
            break;

	    case frmUpdateEvent:
            FrmDrawForm(formPtr);
            DSSDraw();
            handled = true;
            break;

        case popSelectEvent:
            switch (event->data.popSelect.listID) {
                case l_Survey:
                    if (0 <= event->data.popSelect.selection &&
                        StrCompare(userPrefs.survey,
                            LstGetSelectionText(GetObjectPtr(l_Survey),
                                event->data.popSelect.selection))) {
                        StrCopy(userPrefs.survey,
                            LstGetSelectionText(GetObjectPtr(l_Survey),
                                event->data.popSelect.selection));
                        FrmUpdateForm(f_DSSImage, frmRedrawUpdateCode);
                    }
                    break;
                default:
                    break;
            }
            break;

        default:
            handled = false;
            break;
    }

    return handled;
}
