/*
 * Astro Info - a an astronomical calculator/almanac for PalmOS devices.
 * 
 * $Id: AstroGlue.c 735 2008-01-14 04:33:14Z arpruss $
 * Copyright (C) 2001, Astro Info SourceForge Project
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/* Give us access to table data structures. 
 * As a table contains a field, we also need internal field structures.
 *
 * To ease debugging we also want to access internals of other data
 * structures.
 */
#define ALLOW_ACCESS_TO_INTERNALS_OF_CLIPBOARDS
#define ALLOW_ACCESS_TO_INTERNALS_OF_CONTROLS
#define ALLOW_ACCESS_TO_INTERNALS_OF_FINDPARAMS
#define ALLOW_ACCESS_TO_INTERNALS_OF_FORMS
#define ALLOW_ACCESS_TO_INTERNALS_OF_LISTS
#define ALLOW_ACCESS_TO_INTERNALS_OF_MENUS
#define ALLOW_ACCESS_TO_INTERNALS_OF_PROGRESS
#define ALLOW_ACCESS_TO_INTERNALS_OF_SCROLLBARS

#define ALLOW_ACCESS_TO_INTERNALS_OF_BITMAPS
#define ALLOW_ACCESS_TO_INTERNALS_OF_FONTS
#define ALLOW_ACCESS_TO_INTERNALS_OF_WINDOWS

#define ALLOW_ACCESS_TO_INTERNALS_OF_FIELDS
#define ALLOW_ACCESS_TO_INTERNALS_OF_TABLES
#include <PalmOS.h>
#include "AstroGlue.h"
#include "PalmUtil.h"

UInt32 GlueNullTick = 0xffffffff;
extern Boolean EvtGlueSetNullEventTick(UInt32 tick) {
    if (CheckMinRomVersion(sysVersion35))
        return EvtSetNullEventTick(tick);
    else {
        if (GlueNullTick > tick) {
            GlueNullTick = tick;
            return true;
        }
        return false;
    }
}


void WinGlueSetForeColorRGB(const RGBColorType* newColor, RGBColorType* oldColor)
{
    if (CheckMinRomVersion(sysVersion40)) {
        WinSetForeColorRGB(newColor, oldColor);
    }
    else if (CheckMinRomVersion(sysVersion35)) {
        IndexedColorType oldColorIndex;

        if (newColor == NULL) {
            oldColorIndex = WinSetForeColor(0);
            WinSetForeColor(oldColorIndex);
        }
        else {
            oldColorIndex = WinSetForeColor(WinRGBToIndex(newColor));
        }

        if (oldColor != NULL) {
            WinIndexToRGB(oldColorIndex, oldColor);
        }
    }
}



void WinGlueSetTextColorRGB(const RGBColorType* newColor, RGBColorType* oldColor)
{
    if (CheckMinRomVersion(sysVersion40)) {
        WinSetTextColorRGB(newColor, oldColor);
    }
    else if (CheckMinRomVersion(sysVersion35)) {
        IndexedColorType oldColorIndex;

        if (newColor == NULL) {
            oldColorIndex = WinSetTextColor(0);
            WinSetTextColor(oldColorIndex);
        }
        else {
            oldColorIndex = WinSetTextColor(WinRGBToIndex(newColor));
        }

        if (oldColor != NULL) {
            WinIndexToRGB(oldColorIndex, oldColor);
        }
    }
}



void WinGlueSetBackColorRGB(const RGBColorType* newColor, RGBColorType* oldColor)
{
    if (CheckMinRomVersion(sysVersion40)) {
        WinSetBackColorRGB(newColor, oldColor);
    }
    else if (CheckMinRomVersion(sysVersion35)) {
        IndexedColorType oldColorIndex;

        if (newColor == NULL) {
            oldColorIndex = WinSetBackColor(0);
            WinSetBackColor(oldColorIndex);
        }
        else {
            oldColorIndex = WinSetBackColor(WinRGBToIndex(newColor));
        }

        if (oldColor != NULL) {
            WinIndexToRGB(oldColorIndex, oldColor);
        }
    }
}



