/*
 * Astro Info - an astronomical visualizer for PalmOS devices.
 * 
 * $Id: AstroMatrix.c 708 2008-01-12 15:20:02Z arpruss $
 * Copyright (C) 2002, Astro Info SourceForge Project
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#include <PalmOS.h>
#include "AstroMatrix.h"
#include "FixedPoint.h"

/**
 * Normalize v, such that ||v||=1.
 */
void VecNormalize (VectorType v)
{
    Int32 len;
    len = fpsq(v[0]) + fpsq(v[1]) + fpsq(v[2]);
    len = fpsqrt(len);

    /* normalize vector */
    v[0] = fpdiv(v[0], len);
    v[1] = fpdiv(v[1], len);
    v[2] = fpdiv(v[2], len);
}

void MatVecMul (VectorType dest, MatrixType m, VectorType v)
{
    Int16 i;
    VectorType *row  = &m[0];
    Int32      *dptr = &dest[0];
    for (i = 0; i < 3; i++) {
        *dptr = VecScalarProd(*row, v);
        dptr++;
        row++;
    }
}

void MatRotate(MatrixType dest, Int32 lat, Int32 lng)
{
    Int32 sinlat, coslat, sinlng, coslng;
    fpsincos(lat, &sinlat, &coslat);
    fpsincos(lng, &sinlng, &coslng);
    dest[0][0] = coslng;
    dest[0][1] = sinlng;
    dest[0][2] = 0;
    dest[1][0] = -fpmul(coslat, sinlng);
    dest[1][1] =  fpmul(coslat, coslng);
    dest[1][2] =  sinlat;
    dest[2][0] =  fpmul(sinlat, sinlng);
    dest[2][1] = -fpmul(sinlat, coslng);
    dest[2][2] =  coslat;
}

void MatTranspose(MatrixType m) {
    Int32 t;
    t = m[0][1];
    m[0][1] = m[1][0];
    m[1][0] = t;
    t = m[0][2];
    m[0][2] = m[2][0];
    m[2][0] = t;
    t = m[1][2];
    m[1][2] = m[2][1];
    m[2][1] = t;
}

void MatMatMul (MatrixType dest, MatrixType m1, MatrixType m2)
{
    Int16 r,c;
    for (r = 0; r < 3; r++)
        for (c = 0; c < 3; c++)
            dest[r][c] = (fpmul(m1[r][0],m2[0][c])
                          + fpmul(m1[r][1],m2[1][c])
                          + fpmul(m1[r][2],m2[2][c]));
}

