/*
 * Astro QTree - an astronomical visualizer for PalmOS devices.
 * 
 * $Id: AstroQTreeObject.h 968 2010-04-11 13:31:28Z hoenicke $
 * Copyright (C) 2001, Astro Info SourceForge Project
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#ifndef AstroQTreeObject_h
#define AstroQTreeObject_h

typedef struct {
    VectorType radec; /* 0 */
    Int16  mag;       /* 12 */
    UInt16 sx;        /* 14 */
    UInt16 sy;        /* 16 */
    union {
          struct {
              UInt16 type;      /* 18 */
              UInt16 index;     /* 20 */
          } typeIndex;
          float distance;
    } extraData;
} QTreeObject;

typedef struct {
    UInt8 radecData[3*3];
    UInt8 mag;
} QTreeObjectSmall;

typedef struct {
    UInt8 radecData[3*3];
    UInt8 mag[2];
} QTreeObjectMedium;

/* It would seem simpler to use sizeof(QTreeObjectSize), but in ARM
   code that would force 4-byte padding, which we do not want, since
   this has to work with 68K structures. */
#define SIZE_QTreeObjectSmall (sizeof(UInt8)*(3*3+1))
#define SIZE_QTreeObjectMedium (sizeof(UInt8)*(3*3+2))

#define V2_SIZE_QTreeObject (sizeof(VectorType)+5*sizeof(Int16))
#define SIZE_QTreeObject_up_to_mag (sizeof(VectorType)+sizeof(Int16))
#define SIZE_QTreeObject_up_to_sx  (sizeof(VectorType)+2*sizeof(Int16))
#define SIZE_QTreeObject_up_to_sy  (sizeof(VectorType)+3*sizeof(Int16))
#define SIZE_QTreeObject_up_to_extraData (sizeof(VectorType)+5*sizeof(Int16))

#ifdef ARM
# define NET16(x)  ((((UInt16)(x)&0xFF) << 8) | ((UInt16)(x) >> 8))
#else
# define NET16(x)  (x)
#endif

#define MAGNITUDE(size,p,magHigh) (Int16)((size)==SIZE_QTreeObjectSmall ? \
                               (((QTreeObjectSmall*)(p))->mag | (magHigh)) : \
                               ((size)==SIZE_QTreeObjectMedium ? \
                                ((UInt16)((QTreeObjectMedium*)(p))->mag[0] << 8) | \
                                ((QTreeObjectMedium*)(p))->mag[1] : \
                                NET16(((QTreeObject*)(p))->mag)))

#endif
