/*
 * Astro QTree - an astronomical visualizer for PalmOS devices.
 *
 * $Id: AstroQTreePno.h 967 2010-04-07 07:44:22Z hoenicke $
 * Copyright (C) 2001, Astro Info SourceForge Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef AstroQTreePno_h
#define AstroQTreePno_h

#define RESOURCE_NUM_QTREE_PNO 1
#define RESOURCE_TYPE_PNO      'armc'

#include "AstroMatrix.h"
#include "Astro.h"
#include "AstroQTreeObject.h"
#include "AstroPnoUtil.h"
#include "AstroHorizon.h"

/* A number of these items get translated from 68k format
   to ARM format the first time this routine is called. */
typedef struct {
    /* 4-byte aligned items */
    MatrixType   rotate;  /* <-caller(68k format) */
    VectorType   zenith;  /* <-caller(68k format) */
    QTreeObject* ptr;     /* <-caller(68k format) */
    UInt8*       bits;          /* <-caller(68k format) */
    DBRecord*    catDB;         /* <-caller(68k format) */
    UInt16*      colors;        /* <-caller(pointer in ARM format(!),
                                     data in 68k format) */
    UInt16*      fadedColors;        /* <-caller(pointer in ARM format(!),
                                     data in 68k format) */
    Int32        minDec, maxDec;
    Int32        minRa, maxRa;
    Int32        xMin;    /* internal */
    Int32        xMax;    /* internal */
    Int32        yMin;    /* internal */
    Int32        yMax;    /* internal */

    /* 2-byte aligned items */
    UInt16     rootIdx;   /* <-caller(68k format) */
    UInt16     fadeOffset; /* <-caller(68k format) */
    Int16      maxMag;    /* <-caller(68k format) */
    Int16      skyHeight; /* <-caller(68k format) */
    Int16      skyWidth;  /* <-caller(68k format) */
    Int16      rowBytes;  /* <-caller(68k format) */
    UInt16     colorNum;  /* <-caller(68k format) */

    /* 1-byte aligned items */
    Int8       zoomValue;   /* <-caller */
    UInt8      numObj;      /* <-caller */
    UInt8      qTreeObjectSize;  /* <-caller(68k format) */
    Int8       maxBrightnessForDrawingCode; /* <-caller */
    UInt8      maxBrightnessForColorList;   /* <-caller */
    UInt8      depth;        /* <-caller */
    UInt8      horizon;      /* <-caller */
    UInt8      magHigh;      /* <-caller */
} QTreePnoDataType;

typedef struct {
    /* 4-byte aligned */
    MemHandle  handle;
    void*      function;
    QTreePnoDataType data;
} QTreePnoType;

#endif
