/*
 * Astro Info - a an astronomical calculator/almanac for PalmOS devices.
 * 
 * $Id: AstroRocker.c 930 2009-01-30 19:21:09Z arpruss $
 * Copyright (C) 2002, Astro Info SourceForge Project
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <PalmOS.h>
#include "AstroRocker.h"
#include "Astro.h"
#include <HostControl.h>

RockerType * AROCKERSECTION
SetRockerButtons(RockerType *rocker, UInt16 rockupid, UInt16 rockdownid)
{
    FormPtr formPtr = FrmGetActiveForm();

    rocker->rockupid = rockupid;
    rocker->rockdownid = rockdownid;

    // Hide rocker and get out if there is only one page.
    if (rocker->minValue >= rocker->maxValue) {
        HostTraceOutputTL(appErrorClass, "SetRockerButtons hiding");
        FrmHideObject(formPtr, FrmGetObjectIndex(formPtr, rocker->rockdownid));
        FrmHideObject(formPtr, FrmGetObjectIndex(formPtr, rocker->rockupid));
        return rocker;
    } else {
        HostTraceOutputTL(appErrorClass, "SetRockerButtons showing");
        FrmShowObject(formPtr, FrmGetObjectIndex(formPtr, rocker->rockdownid));
        FrmShowObject(formPtr, FrmGetObjectIndex(formPtr, rocker->rockupid));
    }
    
    // Show disabled up arrow if already at top
    if (rocker->Value == rocker->minValue)
        SetControlLabel(rocker->rockupid, "\010");
    else
        SetControlLabel(rocker->rockupid, "\006");
    
    if (rocker->Value == rocker->maxValue)
        SetControlLabel(rocker->rockdownid, "\011");
    else
        SetControlLabel(rocker->rockdownid, "\007");
    
    return rocker;
}


RockerType * AROCKERSECTION
InitRocker(RockerType *rocker, Int16 minvalue, Int16 value, Int16 maxvalue)
{
    rocker->minValue = minvalue;
    rocker->maxValue = maxvalue;
    rocker->zoom     = false;

    HostTraceOutputTL(appErrorClass, "InitRocker %hd %hd %hd", rocker->minValue, value, rocker->maxValue);

    SetRocker(rocker, value);

    return rocker;
}



void AROCKERSECTION
SetRocker(RockerType *rocker, Int16 value)
{
    // Change the value while keeping it within bounds
    rocker->Value = (value < rocker->minValue) ? rocker->minValue : ((value > rocker->maxValue) ? rocker->maxValue : value);

    HostTraceOutputTL(appErrorClass, "SetRocker %hd %hd %hd", rocker->minValue, rocker->Value, rocker->maxValue);
}



Boolean AROCKERSECTION
ChangeRocker(RockerType *rocker, Boolean rockup)
{
    if (rocker->minValue >= rocker->maxValue)
        return false;

    if (rockup) {
        // If we haven't hit the top.
        if (rocker->Value > rocker->minValue) {
            // If we move away from bottom then show enabled down.
            if (rocker->Value == rocker->maxValue)
                SetControlLabel(rocker->rockdownid, "\007");
                SetRocker(rocker, rocker->Value - 1);
            // If we move to the top then show disabled up.
            if (rocker->Value == rocker->minValue)
                SetControlLabel(rocker->rockupid, "\010");

            return true;
        }
    } else {
        // If we haven't hit the bottom.
        if (rocker->Value < rocker->maxValue) {
            // If we move away from top then show enabled up.
            if (rocker->Value == rocker->minValue)
                SetControlLabel(rocker->rockupid, "\006");
                SetRocker(rocker, rocker->Value + 1);
            // If we move to the bottom then show disabled down.
            if (rocker->Value == rocker->maxValue)
                SetControlLabel(rocker->rockdownid, "\011");
            
            return true;
        }
    }

    return false;
}



/*
 * Rocker code based on switching two pairs of characters representing up/down enabled/disabled arrows.
 */
Boolean AROCKERSECTION
RockerChangeHandleEvent(RockerType *rocker, EventPtr event)
{
    Boolean handled = false, changed = false;

    switch (event->eType) {
    case keyDownEvent:
        if (userPrefs.pageBtnFunction ==  PAGEBTNROCKER ||
            (userPrefs.pageBtnFunction == PAGEBTNZOOM && rocker->zoom)) {
            if (event->data.keyDown.chr == pageUpChr) {
                changed = ChangeRocker(rocker, true);
                handled = true;
                break;

            }
            else if (event->data.keyDown.chr == pageDownChr) {
                changed = ChangeRocker(rocker, false);
                handled = true;
                break;
            }
        }
        break;

    case ctlRepeatEvent:
        if (event->data.ctlRepeat.controlID == rocker->rockupid) {
            changed = ChangeRocker(rocker, true);
            handled = true;
        }

        if (event->data.ctlRepeat.controlID == rocker->rockdownid) {
            changed = ChangeRocker(rocker, false);
            handled = true;
        }
        break;

    default:
    }

    return changed;
}
