/*
 * Astro Info - a an astronomical calculator/almanac for PalmOS devices.
 * 
 * $Id: CityTime.h 613 2006-08-21 18:24:39Z hoenicke $
 * Copyright (C) 2001, Astro Info SourceForge Project
 * CityTime structures Copyright (C) 2001, Code City
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#ifndef CityTime_h
#define CityTime_h

#define CITYSECTION __attribute__ ((section ("code2")))

/* CityTime support for AstroInfo. */

/* Start of code by Code City
 *
 * The structures defined here were kindly provided by Darren Beck
 * <DarrenB@codecity.net>.
 */

/* structures for coordinates */

/* Lat / Lon Coordinates */
typedef struct
{
    Int16 deg;     /* -ve degrees are North and West, +ve degrees
                    * are South and East */
    Int16 min;     /* same for these to cover 0 deg xx mins West
                    * problem. -ve is Nth or Wst */
} degMinType;

typedef struct
{
    degMinType lat;
    degMinType lon;

    /* zero, if coord were selected by clicking on the map */
    Int16 exact;
} coordType;

/* structures for dst */

typedef struct
{
    
    Int16 weekNo;   /* (1-first, 2-2nd, 3-3rd, 4-4th, 5-last) */
    Int16 weekDay;  /* 0-Sunday, 6-Saturday */
    Int16 month;    /* 0-jan, 1-feb, 2-mar ... 11-Dec */
} dstStartStopType;

typedef struct
{
    dstStartStopType start;
    dstStartStopType stop;
    Int16 dstManual;   /* 0-manual off, 1-manual on,
                        * 2-auto (was off), 3-auto (was on)
                        */
    Int16 installVers; /* version to install with */
} dstType;

#define cityNameChars 20

/* Structure of database records. */
typedef struct
{
    Char      cityName [cityNameChars + 1];
    Int16     timeFromGmt;  // times 4 to allow for 1/4hr offsets
    Int16     defaultCity;  // spare - not being used at the moment
    coordType locn;
    dstType   dst;
} CityRecordType;

typedef CityRecordType *CityRecordTypePtr;


/* version 3.0 and earlier preference type */
typedef struct
{
    Int16   savedCity[5];
    Int16   version;
    Boolean toggle[10];
    Int16   ints[4];
} CityTime30PreferenceType;

/* app defines */
#define CityTimeAppType   'CiAa'
#define CityTimePrefID    0       /* preferences resource ID. */

/* database defines */
#define CityTimeDBType   'DATA'   /* type for application database. */
#define CityTimeDBName   "CityTimeDB"  /* name for cities database.
                                        * up to 31 characters. */

/* End of code by Code City */


UInt32 CITYSECTION CityOpen();
void CITYSECTION CityClose();
void CITYSECTION CitySetCity(Int16 recNr);
void CITYSECTION CityListDrawData(Int16 itemNum, RectangleType *bounds, 
                                  Char **itemsText);
UInt16 CITYSECTION CityGetPrefs();

#endif
