/*
 * Astro Info - a an astronomical calculator/almanac for PalmOS devices.
 * 
 * $Id: PalmUtil.h 922 2009-01-14 13:40:16Z arpruss $
 * Copyright (C) 2001, Astro Info SourceForge Project
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#ifndef PalmUtil_h
#define PalmUtil_h


#include <PalmOS.h>

// Some basic ROM versions
#define sysVersion10	sysMakeROMVersion(1,0,0,sysROMStageRelease,0)
#define sysVersion20	sysMakeROMVersion(2,0,0,sysROMStageRelease,0)
#define sysVersion30	sysMakeROMVersion(3,0,0,sysROMStageRelease,0)
#define sysVersion31	sysMakeROMVersion(3,1,0,sysROMStageRelease,0)
#define sysVersion33	sysMakeROMVersion(3,3,0,sysROMStageRelease,0)
#define sysVersion35	sysMakeROMVersion(3,5,0,sysROMStageRelease,0)
#define sysVersion40	sysMakeROMVersion(4,0,0,sysROMStageRelease,0)

// For writing to databases.
#define offsetof(type, member) (UInt32)&(((type *)0)->member)

/*
 * Returns true if the ROM version is at least
 * minVersion.
 */
#define CheckMinRomVersion(minVersion) (minVersion <= romVersion)

extern UInt32 romVersion;
extern UInt16 ticksPerSec;

/*
 * Not the best place for this, but it used by a number of other files.
 *
 * Max length of all text fields
 */
#define FIELDLENGTH     32

void GotoApplication(UInt32 ApplId);
void *FrmGetObjectPtrByID(FormPtr formPtr, UInt16 objid);

void UpdateTextField(UInt16 objid, Char *text);
void SetTextLabel(UInt16 objid, Char *t);
void SetTextLabelAligned(UInt16 objid, Char *t, Int16 horzpos, JustificationType align);
void SetControlLabel(UInt16 objid, Char *t);
void SetControlValue(UInt16 objid, Int16 num);
Int16 GetControlValue(UInt16 objid);
void SetFieldValue(UInt16 objid, Int16 num);
void SetNumField(FormPtr formPtr, UInt16 id, Int16 value);
Char * GetStringResource(UInt16 stringid, Char *buffer);
void NightMode(void);
void NightModeCleanup(void);

void drawChars(Char *str,Int32 x,Int32 y, JustificationType align, Boolean black);
Char* TrimName(Char* source, const Char* prefix);

Int16 GetNumField(FormPtr formPtr, UInt16 id);
Boolean InteractionMode(void);
void Show(UInt16 id, Boolean state);
UInt16 WinGlueSetCoordinateSystem(UInt16 newSystem);

#define GetObjectPtr(id) FrmGetObjectPtrByID(FrmGetActiveForm(), (id))
#define GetObjectIndex(id) FrmGetObjectIndex(FrmGetActiveForm(), (id))

#endif    // PalmUtil_h

